@@ -20,17 +20,17 @@
 
 #include "docmetainfo.h"
 
-#include <QRegExp>
-#include <QDebug>
+#include "khc_debug.h"
 
 #include <KConfig>
 #include <KConfigGroup>
 
-#include "htmlsearch.h"
 #include "docentrytraverser.h"
 #include <KLocalizedString>
 #include <ki18n_version.h>
 
+#include <prefs.h>
+
 using namespace KHC;
 
 bool DocMetaInfo::mLoaded = false;
@@ -45,16 +45,14 @@ DocMetaInfo *DocMetaInfo::self()
 
 DocMetaInfo::DocMetaInfo()
 {
-  qDebug() << "DocMetaInfo()";
-
-  mHtmlSearch = new HTMLSearch;
+  khcDebug() << "DocMetaInfo()";
 
   mRootEntry.setName( i18n("Top-Level Documentation") );
 }
 
 DocMetaInfo::~DocMetaInfo()
 {
-  qDebug() << "~DocMetaInfo()";
+  khcDebug() << "~DocMetaInfo()";
 
   DocEntry::List::ConstIterator it;
   for( it = mDocEntries.constBegin(); it != mDocEntries.constEnd(); ++it )
@@ -62,16 +60,13 @@ DocMetaInfo::~DocMetaInfo()
     delete *it;
   }
 
-  delete mHtmlSearch;
-
   mLoaded = false;
 
   mSelf = 0;
 }
 
-DocEntry *DocMetaInfo::addDocEntry( const QString &fileName )
+DocEntry *DocMetaInfo::addDocEntry( const QFileInfo &fi )
 {
-  QFileInfo fi( fileName );
   if ( !fi.exists() ) return 0;
 
   QString extension = fi.completeSuffix();
@@ -82,28 +77,30 @@ DocEntry *DocMetaInfo::addDocEntry( const QString &fileName )
     lang = extensions[ extensions.count() - 2 ];
   }
 
-  if ( !lang.isEmpty() && !mLanguageNames.contains( lang ) )
+  if ( !lang.isEmpty() && !mLanguages.contains( lang ) )
   {
     return 0;
   }
 
   DocEntry *entry = new DocEntry();
 
-  if ( entry->readFromFile( fileName ) ) 
+  if ( entry->readFromFile( fi ) )
   {
-    if ( !lang.isEmpty() && lang != mDefaultLanguage )
+    if ( !lang.isEmpty() && lang != mLanguages.first() )
     {
+      QMap<QString,QString>::Iterator it = mLanguageNames.find( lang );
+      if ( it == mLanguageNames.end() )
+      {
+        it = mLanguageNames.insert( lang, languageName( lang ) );
+      }
+
       entry->setLang( lang );
       entry->setName( i18nc("doctitle (language)","%1 (%2)",
                              entry->name() ,
-                             mLanguageNames[ lang ] ) );
-    }
-    if ( entry->searchMethod().toLower() == "htdig" ) 
-    {
-      mHtmlSearch->setupDocEntry( entry );
+                             *it ) );
     }
     QString indexer = entry->indexer();
-    indexer.replace( "%f", fileName );
+    indexer.replace( "%f", fi.absoluteFilePath() );
     entry->setIndexer( indexer );
     addDocEntry( entry );
     return entry;
@@ -121,12 +118,12 @@ void DocMetaInfo::addDocEntry( DocEntry *entry )
   if ( !entry->search().isEmpty() ) mSearchEntries.append( entry );
 }
 
-DocEntry::List DocMetaInfo::docEntries()
+DocEntry::List DocMetaInfo::docEntries() const
 {
   return mDocEntries;
 }
 
-DocEntry::List DocMetaInfo::searchEntries()
+DocEntry::List DocMetaInfo::searchEntries() const
 {
   return mSearchEntries;
 }
@@ -138,7 +135,7 @@ QString DocMetaInfo::languageName( const QString &langcode )
 
   QString cfgfile = QStandardPaths::locate( QStandardPaths::GenericDataLocation, QStringLiteral( "locale/%1/kf5_entry.desktop" ).arg( langcode ) );
 
-  qDebug() << "-- langcode: " << langcode << " cfgfile: " << cfgfile;
+  khcDebug() << "-- langcode: " << langcode << " cfgfile: " << cfgfile;
 
   KConfig _cfg( cfgfile, KConfig::SimpleConfig );
   KConfigGroup cfg(&_cfg, "KCM Locale" );
@@ -152,21 +149,14 @@ void DocMetaInfo::scanMetaInfo( bool force )
   if ( mLoaded && !force ) return;
 
 #if KI18N_VERSION >= QT_VERSION_CHECK(5, 20, 0)
-  QStringList langs = KLocalizedString::languages();
+  mLanguages = KLocalizedString::languages();
 #else
-  QStringList langs = QLocale().uiLanguages();
+  mLanguages = QLocale().uiLanguages();
 #endif
-  mDefaultLanguage = langs.first();
 
   QStringList::ConstIterator it;
-  for( it = langs.constBegin(); it != langs.constEnd(); ++it )
-  {
-    mLanguageNames.insert( *it, languageName( *it ) );
-  }
 
-  KConfig config( QLatin1String("khelpcenterrc") );
-  KConfigGroup cg(&config, "General");
-  QStringList metaInfos = cg.readEntry( "MetaInfoDirs" , QStringList() );
+  QStringList metaInfos = Prefs::metaInfoDirs();
 
   if ( metaInfos.isEmpty() ) 
   {
@@ -174,7 +164,7 @@ void DocMetaInfo::scanMetaInfo( bool force )
   }
   for( it = metaInfos.constBegin(); it != metaInfos.constEnd(); ++it)
   {
-    qDebug() << "DocMetaInfo::scanMetaInfo(): scanning " << *it;
+    khcDebug() << "DocMetaInfo::scanMetaInfo(): scanning " << *it;
     scanMetaInfoDir( *it, &mRootEntry );
   }
 
@@ -197,7 +187,7 @@ DocEntry *DocMetaInfo::scanMetaInfoDir( const QString &dirName,
     } 
     else if ( fi.suffix() == QLatin1String("desktop") )
     {
-      entry = addDocEntry( fi.absoluteFilePath() );
+      entry = addDocEntry( fi );
       if ( parent && entry ) parent->addChild( entry );
     }
   }
@@ -207,7 +197,7 @@ DocEntry *DocMetaInfo::scanMetaInfoDir( const QString &dirName,
 
 DocEntry *DocMetaInfo::addDirEntry( const QDir &dir, DocEntry *parent )
 {
-  DocEntry *dirEntry = addDocEntry( dir.absolutePath() + QLatin1String("/.directory") );
+  DocEntry *dirEntry = addDocEntry( dir.absoluteFilePath( QLatin1String(".directory") ) );
 
   if ( !dirEntry ) 
   {
@@ -251,7 +241,7 @@ void DocMetaInfo::traverseEntry( DocEntry *entry, DocEntryTraverser *traverser )
 
 void DocMetaInfo::startTraverseEntries( DocEntryTraverser *traverser )
 {
-  qDebug() << "DocMetaInfo::startTraverseEntries()";
+  khcDebug() << "DocMetaInfo::startTraverseEntries()";
   traverser->setNotifyee( this );
   startTraverseEntry( &mRootEntry, traverser );
 }
@@ -261,14 +251,13 @@ void DocMetaInfo::startTraverseEntry( DocEntry *entry,
 {
   if ( !traverser ) 
   {
-    qDebug() << "DocMetaInfo::startTraverseEntry(): ERROR. No Traverser."
-              << endl;
+    khcWarning() << "DocMetaInfo::startTraverseEntry(): ERROR. No Traverser.";
     return;
   }
 
   if ( !entry ) 
   {
-    qDebug() << "DocMetaInfo::startTraverseEntry(): no entry.";
+    khcWarning() << "DocMetaInfo::startTraverseEntry(): no entry.";
     endTraverseEntries( traverser );
     return;
   }
@@ -316,11 +305,11 @@ void DocMetaInfo::endProcess( DocEntry *entry, DocEntryTraverser *traverser )
 
 void DocMetaInfo::endTraverseEntries( DocEntryTraverser *traverser )
 {
-  qDebug() << "DocMetaInfo::endTraverseEntries()";
+  khcDebug() << "DocMetaInfo::endTraverseEntries()";
 
   if ( !traverser ) 
   {
-    qDebug() << " no more traversers.";
+    khcDebug() << " no more traversers.";
     return;
   }
 
